@@ -19,10 +19,11 @@
 #include <glib/gi18n.h>
 
 #include "math-buttons.h"
+#include "math-converter.h"
+#include "math-variable-popup.h"
 #include "financial.h"
 #include "currency.h"
 #include "mp-serializer.h"
-#include "units.h"
 
 enum {
     PROP_0,
@@ -40,7 +41,9 @@ struct MathButtonsPrivate
 
     ButtonMode mode;
     gint programming_base;
-  
+
+    MathConverter *converter;
+
     GtkBuilder *basic_ui, *advanced_ui, *financial_ui, *programming_ui;
 
     GdkColor color_numbers, color_action, color_operator, color_function, color_memory, color_group;
@@ -55,11 +58,6 @@ struct MathButtonsPrivate
     GList *superscript_toggles;
     GList *subscript_toggles;
 
-    GtkWidget *convert_from_combo;
-    GtkWidget *convert_to_combo;
-    MpSerializer *units_serializer;
-    GtkWidget *convert_result_label;
-
     GtkWidget *base_combo;
     GtkWidget *base_label;
     GtkWidget *bit_panel;
@@ -234,12 +232,9 @@ static ButtonData button_data[] = {
     {"end_group",          ")", GROUP,
       /* Tooltip for the end group button */
       N_("End Group [)]")},
-    {"store", NULL, MEMORY,
-      /* Tooltip for the assign variable button */
-      N_("Assign Variable")},
-    {"recall", NULL, MEMORY,
-      /* Tooltip for the insert variable button */
-      N_("Insert Variable")},
+    {"memory", NULL, MEMORY,
+      /* Tooltip for the memory button */
+      N_("Memory")},
     {"character", NULL, MEMORY,
       /* Tooltip for the insert character code button */
       N_("Insert Character Code")},
@@ -314,119 +309,6 @@ static char *finc_dialog_fields[][5] = {
     {NULL,        NULL,          NULL,         NULL,         NULL}
 };
 
-#define MAX_UNITS 20
-struct Unit {
-    char *ui_name;
-    char *internal_name;
-};
-
-struct UnitCategory {
-    char *name;
-    struct Unit units[MAX_UNITS];
-};
-
-static struct UnitCategory categories[] = {
-    {N_("Angle Units"),  {
-                          /* Angle unit */
-                          {N_("Degrees"), "degrees"},
-                          /* Angle unit */
-                          {N_("Radians"), "radians"},
-                          /* Angle unit */
-                          {N_("Gradians"), "gradians"},
-                          {NULL, NULL}}},
-    {N_("Length Units"), {
-                          /* Length unit */
-                          {N_("Parsecs"), "parsecs"},
-                          /* Length unit */
-                          {N_("Light Years"), "lightyears"},
-                          /* Length unit */
-                          {N_("Astronomical Units"), "au"},
-                          /* Length unit */
-                          {N_("Nautical Miles"), "nm"},
-                          /* Length unit */
-                          {N_("Miles"), "miles"},
-                          /* Length unit */
-                          {N_("Kilometers"), "kilometers"},
-                          /* Length unit */
-                          {N_("Cables"), "cables"},
-                          /* Length unit */
-                          {N_("Fathoms"), "fathoms"},
-                          /* Length unit */
-                          {N_("Meters"), "meters"},
-                          /* Length unit */
-                          {N_("Yards"), "yards"},
-                          /* Length unit */
-                          {N_("Feet"), "feet"},
-                          /* Length unit */
-                          {N_("Inches"), "inches"},
-                          /* Length unit */
-                          {N_("Centimeters"), "centimeters"},
-                          /* Length unit */
-                          {N_("Millimeters"), "millimeters"},
-                          /* Length unit */
-                          {N_("Micrometers"), "micrometers"},
-                          /* Length unit */
-                          {N_("Nanometers"), "nanometers"},
-                          {NULL, NULL}}},
-    {N_("Area Units"),   {
-                          /* Area unit */
-                          {N_("Hectares"), "hectares"},
-                          /* Area unit */
-                          {N_("Acres"), "acres"},
-                          /* Area unit */
-                          {N_("m²"), "m²"},
-                          /* Area unit */
-                          {N_("cm²"), "cm²"},
-                          /* Area unit */
-                          {N_("mm²"), "mm²"},
-                          {NULL, NULL}}},
-    {N_("Volume Units"), {
-                          /* Volume unit */
-                          {N_("m³"), "m³"},
-                          /* Volume unit */
-                          {N_("Gallons"), "gallons"},
-                          /* Volume unit */
-                          {N_("Liters"), "liters"},
-                          /* Volume unit */
-                          {N_("Quarts"), "quarts"},
-                          /* Volume unit */
-                          {N_("Pints"), "pints"},
-                          /* Volume unit */
-                          {N_("Milliliters"), "milliliters"},
-                          /* Volume unit */
-                          {N_("cm³"), "cm³"},
-                          /* Volume unit */
-                          {N_("mm³"), "mm³"},
-                          {NULL, NULL}}},
-    {N_("Weight Units"), {
-                          /* Weight unit */
-                          {N_("Tonnes"), "tonnes"},
-                          /* Weight unit */
-                          {N_("Kilograms"), "kilograms"},
-                          /* Weight unit */
-                          {N_("Pounds"), "pounds"},
-                          /* Weight unit */
-                          {N_("Ounces"), "ounces"},
-                          /* Weight unit */
-                          {N_("Grams"), "grams"},
-                          {NULL, NULL}}},
-    {N_("Time Units"),   {
-                          /* Time unit */
-                          {N_("Years"), "years"},
-                          /* Time unit */
-                          {N_("Days"), "days"},
-                          /* Time unit */
-                          {N_("Hours"), "hours"},
-                          /* Time unit */
-                          {N_("Minutes"), "minutes"},
-                          /* Time unit */
-                          {N_("Seconds"), "seconds"},
-                          /* Time unit */
-                          {N_("Milliseconds"), "milliseconds"},
-                          /* Time unit */
-                          {N_("Microseconds"), "microseconds"},
-                          {NULL, NULL}}}
-};
 
 MathButtons *
 math_buttons_new(MathEquation *equation)
@@ -553,19 +435,19 @@ update_bit_panel(MathButtons *buttons)
     if (base != 8) {
         if (label->len != 0)
             g_string_append(label, " = ");
-        g_string_append_printf(label, "%lo", bits);
+        g_string_append_printf(label, "%" G_GINT64_MODIFIER "o", bits);
         g_string_append(label, "₈");
     }
     if (base != 10) {
         if (label->len != 0)
             g_string_append(label, " = ");
-        g_string_append_printf(label, "%lu", bits);
+        g_string_append_printf(label, "%" G_GINT64_MODIFIER "u", bits);
         g_string_append(label, "₁₀");
     }
     if (base != 16) {
         if (label->len != 0)
             g_string_append(label, " = ");
-        g_string_append_printf(label, "%lX", bits);
+        g_string_append_printf(label, "%" G_GINT64_MODIFIER "X", bits);
         g_string_append(label, "₁₆");
     }
 
@@ -614,144 +496,11 @@ update_currency_label(MathButtons *buttons)
 }
 
 
-static void
-update_conversion_bar(MathButtons *buttons)
-{
-    MPNumber x, z;
-    gboolean enabled;
-    gchar *label;
-    const gchar *source_units, *target_units;
-    char *source_value, *target_value;
-
-    if (!buttons->priv->convert_result_label)
-        return;
-
-    enabled = math_equation_get_number(buttons->priv->equation, &x);
-
-    source_units = math_equation_get_source_units(buttons->priv->equation);
-    target_units = math_equation_get_target_units(buttons->priv->equation);
-    if (!source_units || !target_units)
-        enabled = FALSE;
-    else if (!units_convert(&x, source_units, target_units, &z))
-        enabled = FALSE;
-
-    gtk_widget_set_sensitive(buttons->priv->convert_result_label, enabled);
-    if (!enabled)
-        return;
-
-    source_value = mp_serializer_to_string(buttons->priv->units_serializer, &x);
-    target_value = mp_serializer_to_string(buttons->priv->units_serializer, &z);
-
-    label = g_strdup_printf("%s %s = %s %s", source_value, source_units, target_value, target_units);
-    gtk_label_set_text(GTK_LABEL(buttons->priv->convert_result_label), label);
-    g_free(source_value);
-    g_free(target_value);
-    g_free(label);
-}
-
-
 static void
 display_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
 {
     update_currency_label(buttons);
     update_bit_panel(buttons);
-    update_conversion_bar(buttons);
-}
-
-
-static void
-convert_from_combobox_changed_cb(GtkWidget *combo, MathButtons *buttons)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-    int typeindex, unitindex, i;
-
-    model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
-    gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter);
-    gtk_tree_model_get(model, &iter, 1, &typeindex, 2, &unitindex, -1);
-
-    model = GTK_TREE_MODEL(gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT));
-    for (i = 0; categories[typeindex].units[i].ui_name != NULL; i++) {
-        if (i == unitindex)
-            continue;
-        gtk_list_store_append(GTK_LIST_STORE(model), &iter);
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, categories[typeindex].units[i].ui_name, 1, typeindex, 2, i, -1);
-    }
-    gtk_combo_box_set_model(GTK_COMBO_BOX(buttons->priv->convert_to_combo), model);
-
-    math_equation_set_source_units(buttons->priv->equation, categories[typeindex].units[unitindex].internal_name);
-
-    gtk_combo_box_set_active(GTK_COMBO_BOX(buttons->priv->convert_to_combo), 0);
-}
-
-
-static void
-source_units_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-
-    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->convert_from_combo));
-    if (!gtk_tree_model_get_iter_first(model, &iter))
-        return;
-    do
-    {
-        GtkTreeIter child_iter;
-
-        if (gtk_tree_model_iter_children(model, &child_iter, &iter))
-        {
-            do
-            {
-                gint i, j;
-
-                gtk_tree_model_get(model, &child_iter, 1, &i, 2, &j, -1);
-                if (strcmp(categories[i].units[j].internal_name, math_equation_get_source_units(equation)) == 0)
-                {
-                    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(buttons->priv->convert_from_combo), &child_iter);
-                    update_conversion_bar(buttons);
-                    return;
-                }             
-            } while (gtk_tree_model_iter_next(model, &child_iter));
-        }
-    } while (gtk_tree_model_iter_next(model, &iter));
-}
-
-
-static void
-convert_to_combobox_changed_cb(GtkWidget *combo, MathButtons *buttons)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-    int category, toindex;
-
-    model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
-    gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter);
-    gtk_tree_model_get(model, &iter, 1, &category, 2, &toindex, -1);
-    math_equation_set_target_units(buttons->priv->equation, categories[category].units[toindex].internal_name);
-}
-
-
-static void
-target_units_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-
-    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->convert_to_combo));
-    if (!gtk_tree_model_get_iter_first(model, &iter))
-        return;
-    do
-    {
-        gint i, j;
-
-        gtk_tree_model_get(model, &iter, 1, &i, 2, &j, -1);
-        if (strcmp(categories[i].units[j].internal_name, math_equation_get_target_units(equation)) == 0)
-        {
-            gtk_combo_box_set_active_iter(GTK_COMBO_BOX(buttons->priv->convert_to_combo), &iter);
-            update_conversion_bar(buttons);
-            return;
-        }
-    } while (gtk_tree_model_iter_next(model, &iter));
 }
 
 
@@ -895,17 +644,6 @@ target_currency_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons
 }
 
 
-static void
-convert_from_cell_data_func(GtkCellLayout   *cell_layout,
-                            GtkCellRenderer *cell,
-                            GtkTreeModel    *tree_model,
-                            GtkTreeIter     *iter,
-                            gpointer         data)
-{
-    g_object_set(cell, "sensitive", !gtk_tree_model_iter_has_child(tree_model, iter), NULL);
-}
-
-
 static GtkWidget *
 load_mode(MathButtons *buttons, ButtonMode mode)
 {
@@ -1042,48 +780,6 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         if (math_equation_get_number_mode(buttons->priv->equation) == SUBSCRIPT)
             gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
     }
-  
-    if (mode == ADVANCED) {
-        GtkTreeStore *from_model;
-        GtkTreeIter iter, parent;
-        GtkCellRenderer *renderer;
-        int i, j;
-
-        buttons->priv->convert_result_label = GET_WIDGET(builder, "convert_result_label");
-        buttons->priv->units_serializer = mp_serializer_new(MP_DISPLAY_FORMAT_AUTOMATIC, 10, 2);
-
-        buttons->priv->convert_from_combo = GET_WIDGET(builder, "convert_from_combo");
-        from_model = gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);
-        gtk_combo_box_set_model(GTK_COMBO_BOX(buttons->priv->convert_from_combo), GTK_TREE_MODEL(from_model));
-
-        for (i = 0; i < sizeof(categories) / sizeof(categories[0]); i++) {
-            gtk_tree_store_append(from_model, &parent, NULL);
-            gtk_tree_store_set(from_model, &parent, 0, categories[i].name, 1, i, -1);
-            for (j = 0; categories[i].units[j].ui_name != NULL; j++) {
-                gtk_tree_store_append(from_model, &iter, &parent);
-                gtk_tree_store_set(from_model, &iter, 0, categories[i].units[j].ui_name, 1, i, 2, j, -1);
-            }
-        }
-        renderer = gtk_cell_renderer_text_new();
-        gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(buttons->priv->convert_from_combo), renderer, TRUE);
-        gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(buttons->priv->convert_from_combo), renderer, "text", 0);
-        gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(buttons->priv->convert_from_combo),
-                                           renderer,
-                                           convert_from_cell_data_func,
-                                           NULL, NULL);
-
-        buttons->priv->convert_to_combo = GET_WIDGET(builder, "convert_to_combo");
-        renderer = gtk_cell_renderer_text_new();
-        gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(buttons->priv->convert_to_combo), renderer, TRUE);
-        gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(buttons->priv->convert_to_combo), renderer, "text", 0);
-
-        g_signal_connect(buttons->priv->convert_from_combo, "changed", G_CALLBACK(convert_from_combobox_changed_cb), buttons);
-        g_signal_connect(buttons->priv->convert_to_combo, "changed", G_CALLBACK(convert_to_combobox_changed_cb), buttons);
-        g_signal_connect(buttons->priv->equation, "notify::source-units", G_CALLBACK(source_units_changed_cb), buttons);
-        g_signal_connect(buttons->priv->equation, "notify::target-units", G_CALLBACK(target_units_changed_cb), buttons);
-        source_units_changed_cb(buttons->priv->equation, NULL, buttons);
-        target_units_changed_cb(buttons->priv->equation, NULL, buttons);
-    }
 
     if (mode == PROGRAMMING) {
         GtkListStore *model;
@@ -1199,6 +895,11 @@ load_buttons(MathButtons *buttons)
     if (!gtk_widget_get_visible(GTK_WIDGET(buttons)))
         return;
 
+    if (!buttons->priv->converter) {
+        buttons->priv->converter = math_converter_new(buttons->priv->equation);
+        gtk_box_pack_start(GTK_BOX(buttons), GTK_WIDGET(buttons->priv->converter), FALSE, TRUE, 0);      
+    }
+
     panel = load_mode(buttons, buttons->priv->mode);
     if (buttons->priv->active_panel == panel)
         return;
@@ -1232,6 +933,8 @@ math_buttons_set_mode(MathButtons *buttons, ButtonMode mode)
 
     load_buttons(buttons);
 
+    gtk_widget_set_visible(GTK_WIDGET(buttons->priv->converter), mode == ADVANCED);
+
     g_object_notify(G_OBJECT(buttons), "mode");
 }
 
@@ -1352,163 +1055,23 @@ popup_button_menu(GtkWidget *widget, GtkMenu *menu)
 }
 
 
-static void
-save_variable_cb(GtkWidget *widget, MathButtons *buttons)
-{
-  printf("save\n");
-}
-
-
-static void
-delete_variable_cb(GtkWidget *widget, MathButtons *buttons)
-{
-  printf("delete\n");
-}
-
-
-static GtkWidget *
-make_register_menu_item(MathButtons *buttons, const gchar *name, const MPNumber *value, gboolean can_modify, GCallback callback)
-{
-    gchar *text, *mstr;
-    GtkWidget *item, *label;
-
-    if (value) {
-        MpSerializer *serializer = math_equation_get_serializer(buttons->priv->equation);
-        text = mp_serializer_to_string(serializer, value);
-        mstr = g_strdup_printf("<span weight=\"bold\">%s</span> = %s", name, text);
-        g_free(text);
-    }
-    else
-        mstr = g_strdup_printf("<span weight=\"bold\">%s</span>", name);
-    label = gtk_label_new(mstr);
-    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    g_free(mstr);
-
-    item = gtk_menu_item_new();
-
-    // FIXME: Buttons don't work inside menus...
-    if (0) {//can_modify) {
-        GtkWidget *hbox, *button;
-
-        hbox = gtk_hbox_new(FALSE, 6);
-        gtk_container_add(GTK_CONTAINER(item), hbox);
-
-        gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
-
-        button = gtk_button_new();
-        gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU));
-        gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-        gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, TRUE, 0);
-        g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(delete_variable_cb), buttons);
-
-        button = gtk_button_new();
-        gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU));
-        gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-        gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, TRUE, 0);
-        g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(save_variable_cb), buttons);
-    }
-    else
-        gtk_container_add(GTK_CONTAINER(item), label);
-
-    g_object_set_data(G_OBJECT(item), "register_id", g_strdup(name)); // FIXME: Memory leak
-    g_signal_connect(item, "activate", callback, buttons);
-  
-    return item;
-}
-
-
-static void
-store_menu_cb(GtkMenuItem *menu, MathButtons *buttons)
-{
-    math_equation_store(buttons->priv->equation, g_object_get_data(G_OBJECT(menu), "register_id"));
-}
-
-
-void store_cb(GtkWidget *widget, MathButtons *buttons);
+void memory_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
-store_cb(GtkWidget *widget, MathButtons *buttons)
+memory_cb(GtkWidget *widget, MathButtons *buttons)
 {
-    int i;
-    GtkWidget *menu;
-    GtkWidget *item;
-    gchar **names;
+    MathVariablePopup *popup;
+    GtkAllocation allocation;
+    gint x, y;
 
-    menu = gtk_menu_new();
-    gtk_menu_set_reserve_toggle_size(GTK_MENU(menu), FALSE);
-    set_tint(menu, &buttons->priv->color_memory, 1);
+    popup = math_variable_popup_new(buttons->priv->equation);
+    set_tint(GTK_WIDGET(popup), &buttons->priv->color_memory, 1);
+    gtk_window_set_transient_for(GTK_WINDOW(popup), GTK_WINDOW(gtk_widget_get_toplevel(widget)));
 
-    names = math_variables_get_names(math_equation_get_variables(buttons->priv->equation));
-    if (names[0] == NULL) {
-        item = gtk_menu_item_new_with_label(/* Text shown in store menu when no variables defined */
-                                            _("No variables defined"));
-        gtk_widget_set_sensitive(item, FALSE);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    }  
-    for (i = 0; names[i]; i++) {
-        MPNumber *value;
-        value = math_variables_get_value(math_equation_get_variables(buttons->priv->equation), names[i]);
-        item = make_register_menu_item(buttons, names[i], value, TRUE, G_CALLBACK(store_menu_cb));
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    }
-
-    g_strfreev(names);
-
-    // FIXME
-    //item = gtk_menu_item_new_with_label(_("Add variable"));
-    //gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-
-    gtk_widget_show_all(menu);
-    popup_button_menu(widget, GTK_MENU(menu));
-}
-
-
-static void
-recall_menu_cb(GtkMenuItem *menu, MathButtons *buttons)
-{
-    math_equation_recall(buttons->priv->equation, g_object_get_data(G_OBJECT(menu), "register_id"));  
-}
-
-
-void recall_cb(GtkWidget *widget, MathButtons *buttons);
-G_MODULE_EXPORT
-void
-recall_cb(GtkWidget *widget, MathButtons *buttons)
-{
-    int i;
-    GtkWidget *menu;
-    GtkWidget *item;
-    gchar **names;
-
-    menu = gtk_menu_new();
-    gtk_menu_set_reserve_toggle_size(GTK_MENU(menu), FALSE);
-    set_tint(menu, &buttons->priv->color_memory, 1);
-
-    names = math_variables_get_names(math_equation_get_variables(buttons->priv->equation));
-    if (names[0] == NULL) {
-        item = gtk_menu_item_new_with_label(/* Text shown in recall menu when no variables defined */
-                                            _("No variables defined"));
-        gtk_widget_set_sensitive(item, FALSE);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    }  
-    for (i = 0; names[i]; i++) {
-        MPNumber *value;
-        value = math_variables_get_value(math_equation_get_variables(buttons->priv->equation), names[i]);
-        item = make_register_menu_item(buttons, names[i], value, TRUE, G_CALLBACK(recall_menu_cb));
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    }
-
-    g_strfreev(names);
-
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
-    item = make_register_menu_item(buttons, "ans", math_equation_get_answer(buttons->priv->equation), FALSE, G_CALLBACK(recall_menu_cb));
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    item = make_register_menu_item(buttons, "rand", NULL, FALSE, G_CALLBACK(recall_menu_cb));
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-
-    gtk_widget_show_all(menu);
-    popup_button_menu(widget, GTK_MENU(menu));
+    gtk_widget_get_allocation(widget, &allocation); 
+    gdk_window_get_root_coords(gtk_widget_get_window(widget), allocation.x, allocation.y, &x, &y);
+    gtk_window_move(GTK_WINDOW(popup), x, y);
+    gtk_widget_show(GTK_WIDGET(popup));
 }
 
 
@@ -1984,6 +1547,7 @@ static void
 math_buttons_init(MathButtons *buttons)
 {
     buttons->priv = G_TYPE_INSTANCE_GET_PRIVATE(buttons, math_buttons_get_type(), MathButtonsPrivate);
+    gtk_box_set_spacing(GTK_BOX(buttons), 6);
     buttons->priv->programming_base = 10;
     gdk_color_parse("#0000FF", &buttons->priv->color_numbers);
     gdk_color_parse("#00FF00", &buttons->priv->color_action);
